﻿//------------------------------------------------------------------------------
// <copyright file="HeaderFlip.cs" company="Company">
//     Copyright (c) Company.  All rights reserved.
// </copyright>
//------------------------------------------------------------------------------

using System;
using System.ComponentModel.Design;
using System.Globalization;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;

using System.Collections.Specialized;

namespace SolutionOpen.Command
{
	/// <summary>
	/// Command handler
	/// </summary>
	internal sealed class HeaderFlip
	{
		/// <summary>
		/// Command ID.
		/// </summary>
		public const int CommandId = 4129;

		/// <summary>
		/// Command menu group (command set GUID).
		/// </summary>
		public static readonly Guid CommandSet = new Guid("f044c1a5-742b-472d-b45a-fc2b5df582f9");

		/// <summary>
		/// VS Package that provides this command, not null.
		/// </summary>
		private readonly Package package;

		/// <summary>
		/// Initializes a new instance of the <see cref="HeaderFlip"/> class.
		/// Adds our command handlers for menu (commands must exist in the command table file)
		/// </summary>
		/// <param name="package">Owner package, not null.</param>
		private HeaderFlip(Package package)
		{
			if(package == null)
			{
				throw new ArgumentNullException("package");
			}

			this.package = package;

			OleMenuCommandService commandService = this.ServiceProvider.GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
			if(commandService != null)
			{
				var menuCommandID = new CommandID(CommandSet, CommandId);
				var menuItem = new MenuCommand(this.MenuItemCallback, menuCommandID);
				commandService.AddCommand(menuItem);
			}
		}

		/// <summary>
		/// Gets the instance of the command.
		/// </summary>
		public static HeaderFlip Instance
		{
			get;
			private set;
		}

		/// <summary>
		/// Gets the service provider from the owner package.
		/// </summary>
		private IServiceProvider ServiceProvider
		{
			get
			{
				return this.package;
			}
		}

		/// <summary>
		/// Initializes the singleton instance of the command.
		/// </summary>
		/// <param name="package">Owner package, not null.</param>
		public static void Initialize(Package package)
		{
			Instance = new HeaderFlip(package);
		}

		/// <summary>
		/// This function is the callback used to execute the command when the menu item is clicked.
		/// See the constructor to see how the menu item is associated with this function using
		/// OleMenuCommandService service and MenuCommand class.
		/// </summary>
		/// <param name="sender">Event sender.</param>
		/// <param name="e">Event args.</param>
		private void MenuItemCallback(object sender, EventArgs e)
		{
			// make sure there is an active document
			if (SolutionOpenPackage.ApplicationObject.ActiveDocument != null)
			{
				// Get the current active document and its base name
				String activeDocumentFullName = SolutionOpenPackage.ApplicationObject.ActiveDocument.FullName;
				String activeDocumentBaseName = SolutionOpenPackage.BaseName(activeDocumentFullName);

				// find the files with the same base name
				StringCollection matchingFilePaths = new StringCollection();
				foreach (String fullName in SolutionOpenPackage.FilePaths.Values)
				{
					String baseName = SolutionOpenPackage.BaseName(fullName);
					if (String.Compare(activeDocumentBaseName, baseName, true) == 0)
						matchingFilePaths.Add(fullName);
				}

				// make sure that we have more than one of them open before we try to switch
				if (matchingFilePaths.Count > 1)
				{
					// figure out which document in the list (if any) is the current document
					int activeDocumentIndex = -1;
					for (int i = 0; activeDocumentIndex < 0 && i < matchingFilePaths.Count; ++i)
					{
						if (String.Compare(activeDocumentFullName, matchingFilePaths[i], true) == 0)
							activeDocumentIndex = i;
					}

					// make sure we found this document
					if (activeDocumentIndex >= 0)
					{
						// figure out which document comes next
						int nextDocument = activeDocumentIndex + 1;
						if (nextDocument >= matchingFilePaths.Count)
							nextDocument = 0;

						// open and display that document
						SolutionOpenPackage.ApplicationObject.ItemOperations.OpenFile(matchingFilePaths[nextDocument], EnvDTE.Constants.vsViewKindAny);
					}
				}
			}
		}
	}
}
